#!/usr/bin/perl

#Script by Bia Walter

#This script receives a fasta file containing substructures on lncRNAs of many species. It then classifies each substructure, or entry of the fasta file, according to the alignment position of the long non-coding molecule.

#$perl assign-orthologous.pl substructures.fasta

use strict;
use Data::Dumper;

my $reading_file = shift;
open ("reading_file", $reading_file) || die "It was not possible to open file $reading_file\n";

my @headers;
my @sequences;

#Storing all substructures in array @structures
while (<reading_file>){
	chomp;
	if (/^>/){
		push @headers, "$_";          
	} else {
		push @sequences, "$_";          
	}
}

close reading_file;

my $species;
my @line;
my @class;
my $x = 0;
my $i = 0;
my @pos;
my $length;
my $sub = 0;
my $marker;


foreach(@headers){
	@line = split /\s+/;
	@pos = split (/-/, $line[4]);	

	$length = length $sequences[$i];

	unless (/\b$headers[0]\b/){
		$x = 1;
	}

	#Working all the first species substructures
	if ($x == 0){
		#Adding the positions of all the substructures to array @class
		$sub++;
		push @class, "$pos[0]-$pos[1]-$sub-$length";
		#Printing all the substructures in the headers of the fasta, and correspondant sequences
		print "$line[0]\t$line[1] $line[2]\t$line[3] $line[4] sub$sub | $line[6]\n";
		print $sequences[$i], "\n";
	#Else works all the subsequent species
	} else {
		$marker = 0;

		#Separating initial and end positions of current sub-structure
		my @data_curr = split (/-/, $line[4]);
		my $thresh = 0.3*$length;

		foreach my $class(@class){
			my @data = split (/-/, $class);

			#Makes the evaluation if the current substructure was already annotated in array @class
			if ($data_curr[0] >= $data[0] - $thresh && $data_curr[0] <= $data[0] + $thresh){
				#If match; then print header and sequence, with substructure annotation being the same of the matched substructure of @class
				print "$line[0]\t$line[1] $line[2]\t$line[3] $line[4] sub$data[2] | $line[6]\n";
				print $sequences[$i], "\n";
				$marker = 1;
				last;
			} 
		}
		#If only applies in case the substructure does not match any of @class
		if ($marker == 0){
			#Adds this new substructure to @class and prints it with a new substructure annotation
			$sub++;
			push @class, "$pos[0]-$pos[1]-$sub-$length";
			print "$line[0]\t$line[1] $line[2]\t$line[3] $line[4] sub$sub | $line[6]\n";
			print $sequences[$i], "\n";
	  } 
	}
	$i++;
}
			

