#!/usr/bin/bash

#Bia Walter
#This shell script models the evolution of an input sequence to contain gaps in varying positions. It starts with an input sequence, which is folded with RNAfold. It then evolves to contain a gap of user-specified length, and is folded with and without the constraint of the ancestral structure. Finally, the structural distance between the un-constrained and constrained structures is calculated with RNAforester. This process is repeated so that the gap will be inserted in every position of the sequence, in a sliding window approach, from the first to the last base of the input sequence. At the end the user will get a summary report containing the information of all evolved sequences and corresponding distance scores. 

#$./gap-modelling.sh input.alg input.out GAP_LENGTH $f4-$species

SPECIES="$1"
OUT="$2"
gap_length="$3"
filename="$4"

echo ">ancestral" >> ancestral.fa;
tail -n1 $SPECIES >> ancestral.fa; 

perl ../scripts/get-structure.pl $OUT > ancestral_structure.txt;

grep -v ">" $SPECIES > tmp
size=$(wc -c <"tmp")
size="$(($size-1))"
end="$(($size-$gap_length+1))"
rm tmp

echo "Gap modelling simulations..."
echo "Iterating over bases: 1 - $end"
echo "RNA_length gap_start gap_length structural_distance" > $filename-$gap_length.indel

for start in `seq 1 $end`;
   do	
	perl ../scripts/evolve.pl ancestral.fa $start $gap_length $start > evolved$start.fa;

	echo ">evolved$start-constrained" > evolved$start-constrained.fa
	grep -v ">" evolved$start.fa >> evolved$start-constrained.fa

	cat evolved$start-constrained.fa > evolved$start-constrained.vienna
	cat ancestral_structure.txt >> evolved$start-constrained.vienna
	
	RNAfold -p -d2 --noLP -C < evolved$start-constrained.vienna > evolved$start-constrained.out; 
	relplot.pl -p evolved${start}-constrained_ss.ps evolved${start}-constrained_dp.ps > evolved${start}-constrained_rss.ps;

	echo ">evolved$start-noconstraint" > evolved$start-noconstraint.fa
	grep -v ">" evolved$start.fa >> evolved$start-noconstraint.fa

	RNAfold -p -d2 --noLP <  evolved$start-noconstraint.fa > evolved$start-noconstraint.out; 
	relplot.pl -p evolved$start-noconstraint_ss.ps evolved$start-noconstraint_dp.ps > evolved$start-noconstraint_rss.ps

	perl ../scripts/get-structure.pl evolved$start-constrained.out > evolved$start-constrained.txt;
	perl ../scripts/get-structure.pl evolved$start-noconstraint.out > evolved$start-noconstraint.txt;

	cat evolved$start-constrained.fa evolved$start-constrained.txt > constraint.vienna
	cat evolved$start-noconstraint.fa evolved$start-noconstraint.txt > noconstraint.vienna

	cat constraint.vienna noconstraint.vienna > comparator.txt

	#RNAforester makes an alignment of the structures and checks similarities/divergence, in the same fashion as CLUSTALW does for a sequence alignment, but taking the structure also into account
	RNAforester -d < comparator.txt > tmp.out

	perl ../scripts/format-output.pl tmp.out $size $start $gap_length >> $filename-$gap_length.indel
	
	rm evolved$start.fa evolved$start-constrained.out evolved${start}-constrained_ss.ps evolved${start}-constrained_dp.ps evolved${start}-constrained_rss.ps evolved$start-noconstraint.fa evolved$start-noconstraint.out evolved$start-noconstraint_ss.ps evolved$start-noconstraint_dp.ps evolved$start-noconstraint_rss.ps evolved$start-noconstraint.txt comparator.txt constraint.vienna noconstraint.vienna evolved$start-constrained.fa evolved$start-constrained.vienna evolved$start-constrained.txt tmp.out

done

rm ancestral.fa ancestral_structure.txt 
 

