#!/usr/bin/perl

#Bia Walter

#This script takes as input a list of observed indels and their correspondent structural impacts along with background information, and returns scores analogous to p-values. The background information is calculated by rank statistics and the gap-modelling pipeline and is composed of a ranked list of `i' gap-impact groups. The present script then checks in which group `n' the observed indels are. Then it calculates the probability `p' of observing them or more extreme indel-cases using the following rule:

# p = (i-n)/i

#with `p' being the probability of observing the indel or a more extreme indel-event
#`i' varying from zero to the number of groups - 1
#`n' being the group the observed indel belongs to

#$perl indel-score.pl $f4-$species-observed.indel $f4-$species > $f4-$species.indel


$reading_file = shift;
open ("reading_file", $reading_file) || die "It was not possible to open file $reading_file\n";

$file_name = shift;

#Iterating over file containing the list of indel structural impacts
while (<reading_file>){

	if (/no_indels/){
		print "no_indels\n";
		exit;
	}

	chomp;
	@row = split (/\t/);
	$indel_length = $row[1]; #indel size
	$impact = $row[3]; #impact

	$n = 0;
	$i = -1;

	#Opening the file with the ranked list of $indel_length gap-length structural impact groups
	open(my $fh, "<", "$file_name-$indel_length.indelgroup") or die "Can't open $file_name-$indel_length.indelgroup\n";

	while(<$fh>){
		$i++;
		
		if(/^(\s+)(.+)/){
			$line = $2;
		} else {
			$line = $_;
		}

		@list_row = split (/\s/, $line);
		#Searching over ranked list
		#If group is found, sets group to variable $i
		if($impact >= $list_row[1]){
			$n = $i;
		}
	}

	if($i == 0){
		$i = 1;
	}

	#Calculating the probability of finding observed indel using the rule p = (i-n)/i, with `i' being the number of groups -1 and `n' being the group the observed indel belongs to
	$score = ($i-$n)/$i;
	
	#If the score is too small: approximating the score to a small number, but big enough for it to be processed further in the pipeline
	if ($score == 0){
		$score = 0.01;
	}
	push (@list,$score);
}

$all = join ',', @list;
print "$all\n";

close reading_file;

